/**
 * Downloads the latest tz data from IANA, then processes it into a usable
 * format ready for inclusion.
 */
module.exports = function (grunt) {
    var
        tmpDir,
        async   = require('async'),
        ftp     = require('ftp-get'),
        tzURL   = 'ftp://ftp.iana.org/tz',
        file    = '/tzdata-latest.tar.gz',
        output  = 'src/tz.js';


    grunt.registerTask('update-tz', 'Get the latest tz data from ftp://ftp.iana.org/', function() {
        var gruntDone = this.async(),
            tasks     = [mktemp, getData, extract, cleanup];

        async.series(tasks, function (err) {
            if (err)
                grunt.log.error(err);

            gruntDone(err);
        });
    });

    function mktemp (taskDone) {
        grunt.log.writeln('making temporary directory...');

        grunt.util.spawn({
            cmd: 'mktemp',
            args: ['-d', '/tmp/grunt.XXXXXXXX']
        }, function (err, results) {
            tmpDir = results.stdout;
            grunt.log.ok('temporary directory:  ' + tmpDir);

            taskDone(err);
        });
    }

    function getData (taskDone) {
        ftp.get(tzURL + file, tmpDir + file, taskDone);
    }

    function extract (taskDone) {
        grunt.log.writeln('extracting ' + tmpDir + file + '...');

        grunt.util.spawn({
            cmd: 'tar',
            args: ['xfz', tmpDir + file],
            opts: { cwd: tmpDir }
        }, function (err, results) {
            grunt.log.ok('done');
            taskDone(err);
        });
    }

    function parseData () {
    }

    function cleanup (taskDone) {
        grunt.util.spawn({
            cmd: 'rm',
            args: ['-rf', tmpDir]
        }, function(err, results) {
            grunt.log.ok('deleted temporary files');
            taskDone(err);
        });
    }
};

